/*!
 *  @file DME.h
 *
 *  @author Daniel Brockmann
 *  @date 13.05.2010
 *
 *  @version 0.1
 *
 *  @brief basic defines, main libraries, prototypes for DME
 *
*/

//! standard libraries
#include <iostream>
#include <string>

//! necessary for childprocess FOLK()
#include <sys/types.h>
#include <unistd.h>

//! necessary for WAITPID()
#include <sys/wait.h>


//! \define ALME_CNT absolute number of AMLE functions that can be received (!!!) by DME
#define ALME_CNT 50

//! \define WLME_CNT absolute number of WMLE functions that can be received (!!!) by DME
#define WLME_CNT 50

//! \define MLME_CNT absolute number of MMLE functions that can be received (!!!) by DME
#define MLME_CNT 50


//! general functions
void BLOCK_RESET(int FKT, char *blk_XLME);
void BLOCK_SET(int FKT, char *blk_XLME);
int BLOCK_CHECK(int FKT, char *blk_XLME);

//! child function prototypes
void ALME_CHILD(char ALME_MSG[], pid_t *pid_ALME, char *blk_ALME, int (*ALME_PRIMITIV)(char MSG[]));
void WLME_CHILD(char WLME_MSG[], pid_t *pid_WLME, char *blk_WLME, int (*WLME_PRIMITIV)(char MSG[]));
//void MLME_CHILD(char MLME_MSG[], pid_t *pid_MLME, char *blk_MLME, int (*MLME_PRIMITIV)(char MSG[]));
//void PLME_CHILD(char PLME_MSG[], pid_t *pid_PLME, char *blk_PLME, int (*PLME_PRIMITIV)(char MSG[]));

//! code convert function prototypes
int ALME_CONVERT(int ALME_CODE);
int WLME_CONVERT(int WLME_CODE);
int MLME_CONVERT(int MLME_CODE);
//int PLME_CONVERT(int PLME_CODE);

//! read function prototypes
void ALME_READ(char ALME_MSG[], pid_t *pid_ALME, char *blk_ALME);
void WLME_READ(char WLME_MSG[], pid_t *pid_WLME, char *blk_WLME);
void MLME_READ(char MLME_MSG[], pid_t *pid_MLME, char *blk_MLME);
//void PLME_READ(char PLME_MSG[], pid_t *pid_PLME, char *blk_PLME);

//! ALME primitive prototypes
int ALME_WMIB_GET_REQUEST(char MSG[]);

//! \TODO complete
void BLOCK_SET(int FKT, char *blk_XLME){
	int FIELD, BIT;

	FIELD=FKT/8;
	BIT=FKT%8;

	//printf ("BLOCK_SET: blk_XLME %x \n", blk_XLME[FIELD]);

	switch (BIT){
		case 0: blk_XLME[FIELD]=blk_XLME[FIELD]|0x01; break;
		case 1: blk_XLME[FIELD]=blk_XLME[FIELD]|0x02; break;
		case 2: blk_XLME[FIELD]=blk_XLME[FIELD]|0x04; break;
		case 3: blk_XLME[FIELD]=blk_XLME[FIELD]|0x08; break;
		case 4: blk_XLME[FIELD]=blk_XLME[FIELD]|0x10; break;
		case 5: blk_XLME[FIELD]=blk_XLME[FIELD]|0x20; break;
		case 6: blk_XLME[FIELD]=blk_XLME[FIELD]|0x40; break;
		case 7: blk_XLME[FIELD]=blk_XLME[FIELD]|0x80; break;
	}

	//printf ("BLOCK_SET: blk_XLME %x \n", blk_XLME[FIELD]);
}

//! \TODO complete
void BLOCK_RESET(int FKT, char *blk_XLME){
	int FIELD, BIT;

	FIELD=FKT/8;
	BIT=FKT%8;

	//printf ("BLOCK_RESET: blk_XLME %x \n", blk_XLME[FIELD]);

	switch (BIT){
		case 0: blk_XLME[FIELD]=blk_XLME[FIELD]&0xFE; break;
		case 1: blk_XLME[FIELD]=blk_XLME[FIELD]&0xFD; break;
		case 2: blk_XLME[FIELD]=blk_XLME[FIELD]&0xFB; break;
		case 3: blk_XLME[FIELD]=blk_XLME[FIELD]&0xF7; break;
		case 4: blk_XLME[FIELD]=blk_XLME[FIELD]&0xEF; break;
		case 5: blk_XLME[FIELD]=blk_XLME[FIELD]&0xDF; break;
		case 6: blk_XLME[FIELD]=blk_XLME[FIELD]&0xBF; break;
		case 7: blk_XLME[FIELD]=blk_XLME[FIELD]&0x7F; break;
	}
	//printf ("BLOCK_RESET: blk_XLME %x \n", blk_XLME[FIELD]);
}

//! \TODO complete
int BLOCK_CHECK(int FKT, char *blk_XLME){
	int FIELD, BIT;

	FIELD=FKT/8;
	BIT=FKT%8;

	if (((blk_XLME[FIELD]>>BIT)&0x01)==1){ return 1;}
	else if (((blk_XLME[FIELD]>>BIT)&0x01)==0) { return 0;}

}
