/*!
 *  @file WLP.cpp
 *
 *  @author Daniel Brockmann
 *  @date 13.05.2010
 *
 *  @version 0.1
 *
 *  @brief monitoring the 3 WLP message queues (WLME, WLPSAP, MACSAP)
 *
 *	There is 1 message queues and 2 data queues necessary for the WLP:
 *	\arg WLME : between \c WLP and \c DME
 *	\arg WLPSAP : between \c WLP and \c Application this is only a DATA queue
 *	\arg MACSAP : between \c WLP and \c MAC this is only a DATA queue
 *
 *	The structure of this file is similar to DME.cpp. Please watch there for further informations.
 *	In addition to that, you should know, that WLP has only one control path from which to receive different commands.
 *	The WLP owns in addition to that 2 data paths. That just should be transmited from the application to the MAC.
 *
 *	The primitives are defined in a separate \e header-file \c WLME_FUNCTIONS.h.
 *
 *
*/


//! needed standard libraries
#include <iostream>
#include <stdlib.h>

//! function prototypes, important defines
#include "DME.h"

//! responsible for message queue init
#include "MSG_BUFFER.h"

//! defines all the different ALME functions (between application and DME)
#include "WLP_FUNCTIONS/WLME_FUNCTIONS.h"

//! global definierte Variablen
WMIB WLPInfoBase;

//! a c++ command for input and output
using namespace std;

//! nothing else than monitoring the 3 different message and data gueues
int main(){

	//! \var buf_* the whole structure that is transmitted and received via message queues
	MSGBUF buf_WLME, buf_WLPSAP, buf_MACSAP;

	//! init the message fields to 0
	memset(buf_WLME.msg,0,SIZE); memset(buf_WLPSAP.msg,0,SIZE); memset(buf_MACSAP.msg,0,SIZE);

	//! \var qid_*: include the ID for the message queues; ID is necessary to get access to the queues
	int qid_WLME, qid_WLPSAP, qid_MACSAP, qid_WLME_P, qid_WLPSAP_P, qid_MACSAP_P;

	//! \var blk_WLME: one list per queue, one field per function, that document whether the function is in use (=1) or not (=0)
	char blk_WLME[WLME_CNT];

	//! init the function - block - lists to 0
	memset(blk_WLME,0,WLME_CNT);

	//! \var pid_WLME: as soon as a child process is created, the childprocess-ID is saved in this fields
	int pid_WLME[WLME_CNT];

	//! \TODO complete
	int STATUS; pid_t CHILDSTATUS;

	//! init the function - pid - lists to 0
	memset(pid_WLME,0,WLME_CNT);

	//! create the 2 different data queues
	DATA_QUEUE_INIT(&qid_WLPSAP, &qid_MACSAP, &qid_WLPSAP_P, &qid_MACSAP_P);

	//! get access to existing message queue that is created by DME
	qid_WLME = msgget(KEY_WLME, 0600);
	qid_WLME_P = msgget(KEY_WLME_P, 0600);

	//! \TODO complete
	//! initialisieren der WMIB Struktur mit Defaultwerden.
	WMIB_DEFAULT_SET;

	//! permanently loop: monitoring the 4 initialised message queues
	while (1){

		//! as soon as a message from DME is received it has to be decode (assigned to the specified function) via WLME_READ()
		if(msgrcv(qid_WLME,&buf_WLME, sizeof(MSGBUF), 2,IPC_NOWAIT)!=-1){

			//! print out in console just for information
			printf("Nachr. von DME empfangen: %s\n",buf_WLME.msg);

			//! assign the incoming message to its special primitiv
			WLME_READ(buf_WLME.msg, pid_WLME, blk_WLME);
		}
		if(msgrcv(qid_WLPSAP,&buf_WLPSAP, sizeof(MSGBUF), 2,IPC_NOWAIT)!=-1){

			//! print out in console just for information
			printf("Informationen von APP empfangen: %s\n",buf_WLPSAP.msg);

			//! assign the incoming message to its special primitiv
			//WLME_READ(buf_WLME.msg[0]);
		}
		if(msgrcv(qid_MACSAP,&buf_MACSAP, sizeof(MSGBUF), 2,IPC_NOWAIT)!=-1){

			//! print out in console just for information
			printf("Informationen von MAC empfangen: %s\n",buf_MACSAP.msg);

			//! assign the incoming message to its special primitiv
			//WLME_READ(buf_WLME.msg[0]);
		}

		//! delete ZOMBIE (defunct) child-processes that were still in list after finished (instead of WAITPID())
		CHILDSTATUS=waitpid(0, &STATUS, WNOHANG);
	}

}

/*
 * start of the function definitions
 * 1. analyse, decode and assign the messages
 * 2. start a child process for each function
 */

/*!void WLME_READ
 * assign the incoming messages from the DME to its
 * corresponding primitiv and starting the assigned childprocess with the correct parameters
 * \param[in] WLME_FKT: number of the primitiv
 * \param[in] pid_t *pid_WLME: pointer to function - process ID - list
 * \param[in] int *blk_WLME: pointer to function - block - list
 */
void WLME_READ(char WLME_MSG[], pid_t *pid_WLME, char *blk_WLME){

	//! \TODO complete description
	int WLME_FKT;

	//! \TODO complete description
	WLME_FKT=WLME_CONVERT((int)WLME_MSG[0]);

	//!switch-case command to separate the different functions and transmit correct parameters to start child process
	switch (WLME_FKT){
		case 0: printf(" WLME_RESET_REQUEST\n"); break;
		case 1: printf(" WLME_STREAM_TRACK_RESPONSE\n"); break;
		case 2: printf(" WLME_WMIB_SET_REQUEST\n"); break;
		case 3: printf(" WLME_WMIB_GET_REQUEST\n");
			//! block NOT necessary, already done by DME
			//! send WLME_CONFIRM_GET_confirm to WLME (DME)
			WLME_CHILD(WLME_MSG, pid_WLME, blk_WLME, &WLME_WMIB_GET_REQUEST); break;
		case 4: printf(" WLME_WSS_CREATE_REQUEST\n"); break;
		case 5: printf(" WLME_WSS_DESTROY_REQUEST\n"); break;
		case 6: printf(" WLME_WSS_ASSOCIATE_REQUEST\n"); break;
		case 7: printf(" WLME_WSS_DISASSOCIATE_REQUEST\n"); break;
		default: printf(" Befehl unbekannt\n");break;
	}
}

/*!int MLME_CONVERT
 * \TODO complete description
 */
int WLME_CONVERT(int WLME_CODE){

	switch (WLME_CODE){
		case 0: return 0;
		case 5: return 1;
		case 8: return 2;
		case 12: return 3;
		case 16: return 4;
		case 20: return 5;
		case 21: return 6;
		case 24: return 7;
		case 29: return 8;
		case 28: return 9;
		default: return -1;
	}

}

/*!void WLME_CHILD
 * checking whether the process/primitiv is already running if not
 * the procedure is creating a child process, sets the blockfield for this function to 1
 * and starting the function in the child process
 * finally it is waiting for the end of the child process, otherwise a so called zombie is rest in the process list
 * \param[in] int WLME_FKT: number of the primitiv
 * \param[in] pid_t *pid_WLME: pointer to function - process ID - list
 * \param[in] int *blk_WLME: pointer to function - block - list
 * \param[in] int (*WLME_PRIMITIV)(): pointer to a function that should start in the child process
 */
void WLME_CHILD(char WLME_MSG[], pid_t *pid_WLME, char *blk_WLME, int (*WLME_PRIMITIV)(char MSG[])){

	//! \TODO complete
	int WLME_FKT;

	//! \var STATUS_WLME_PRIMITIV get the result 0 from the functions, when successfull done
    int STATUS_WLME_PRIMITIV;

    //! \TOTO complete
    WLME_FKT=WLME_CONVERT((int)WLME_MSG[0]);

//    //! if function not already running
//    if (BLOCK_CHECK(WLME_FKT,blk_WLME)==0){
//
//    	//! set function - block - field to 1 that means BLOCKED
//    	BLOCK_SET(WLME_FKT, blk_WLME);

    	/*! create new childprocess folk() and save its ID in function - pid - field
    	 * \NOTE could be expand by case -1 = ERROR
    	 */
		switch (pid_WLME[WLME_FKT] = fork()) {

			//! all command in case 0 are now executed parallel the rest of the program
			case 0:

				//! start selected function and provide message field
				STATUS_WLME_PRIMITIV=(*WLME_PRIMITIV)(WLME_MSG);

				//! exit child process
				exit(0);

			//! all commands in default are executed in regular order
			default:
				break;
		}

//    }
//
//    //! if already running print out message to the user
//    else {
//    	printf("Prozess ist bereits aktiv, bitte erneut versuchen!!\n");
//    }
}






